'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _task_base = require('./base/task_base');

var _task_base2 = _interopRequireDefault(_task_base);

var _deployer = require('../deployer');

var _deployer2 = _interopRequireDefault(_deployer);

var _commandLineArgs = require('command-line-args');

var _commandLineArgs2 = _interopRequireDefault(_commandLineArgs);

var _file = require('../utils/file');

var _deploy_actions = require('../actions/deploy_actions');

var _deploy_actions2 = _interopRequireDefault(_deploy_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DeployTask = function (_TaskBase) {
  (0, _inherits3.default)(DeployTask, _TaskBase);

  function DeployTask(web3, sender) {
    (0, _classCallCheck3.default)(this, DeployTask);

    var _this = (0, _possibleConstructorReturn3.default)(this, (DeployTask.__proto__ || Object.getPrototypeOf(DeployTask)).call(this));

    _this.web3 = web3;
    _this.sender = sender;
    _this.deployActions = new _deploy_actions2.default(new _deployer2.default(_this.web3, _this.sender));
    return _this;
  }

  (0, _createClass3.default)(DeployTask, [{
    key: 'execute',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(args) {
        var _this2 = this;

        var options, head, validatorSet, blockRewards, initialValidators, genesisContracts, contracts, envFile;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                console.log('Deploying contracts. This may take some time...');
                options = this.parseOptions(args);

                if (!(options === null)) {
                  _context.next = 4;
                  break;
                }

                return _context.abrupt('return');

              case 4:
                head = options.head, validatorSet = options.validatorSet, blockRewards = options.blockRewards;

                if (!options.genesis) {
                  _context.next = 17;
                  break;
                }

                initialValidators = Array(3).fill(null).map(function () {
                  return _this2.web3.eth.accounts.create().address;
                });

                console.log('Initial validators are: ' + initialValidators);
                console.log('Deploying genesis contracts');
                _context.next = 11;
                return this.deployActions.deployGenesis(initialValidators);

              case 11:
                genesisContracts = _context.sent;

                console.log('Genesis deployed: ');
                this.prettyPrintAddresses(genesisContracts);
                head = genesisContracts.head.options.address;
                validatorSet = genesisContracts.validatorSet.options.address;
                blockRewards = genesisContracts.blockRewards.options.address;

              case 17:

                if (options.turbo) {
                  console.log('⚡️ Deploying in super speed mode. ⚡️');
                }

                _context.next = 20;
                return this.deployActions.deployAll(head, validatorSet, blockRewards, options.turbo);

              case 20:
                contracts = _context.sent;

                console.log('Contracts deployed: ');
                this.prettyPrintAddresses(contracts);

                envFile = this.contractsToEnvFile(contracts);

                if (!options.save) {
                  _context.next = 29;
                  break;
                }

                _context.next = 27;
                return this.saveEnvfile(options.save, envFile);

              case 27:
                _context.next = 30;
                break;

              case 29:
                this.printSummary(envFile);

              case 30:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function execute(_x) {
        return _ref.apply(this, arguments);
      }

      return execute;
    }()
  }, {
    key: 'parseOptions',
    value: function parseOptions(args) {
      var options = (0, _commandLineArgs2.default)([{ name: 'genesis', type: Boolean }, { name: 'head', type: String }, { name: 'validatorSet', type: String }, { name: 'blockRewards', type: String }, { name: 'turbo', type: Boolean }, { name: 'save', type: String }], { argv: args, partial: true });
      // eslint-disable-next-line no-underscore-dangle
      var unknownOptions = options._unknown;
      if (unknownOptions && unknownOptions.length > 0) {
        console.error('Unknown options: ' + unknownOptions.join(', '));
        return null;
      }

      if (!options.genesis && !options.head) {
        console.error('You should provide a value for the head parameter.');
        return null;
      }

      if (!options.genesis && !options.validatorSet) {
        console.error('You should provide a value for the validatorSet parameter.');
        return null;
      }

      if (!options.genesis && !options.blockRewards) {
        console.error('You should provide a value for the blockRewards parameter.');
        return null;
      }

      if (options.save === null) {
        console.error('You should provide a value for the save parameter.');
        return null;
      }

      return options;
    }
  }, {
    key: 'saveEnvfile',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(envFilePath, envFile) {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;
                _context2.next = 3;
                return (0, _file.writeFile)(envFilePath, envFile);

              case 3:
                console.log('Contracts deployed, env saved to ' + envFilePath + '.');
                _context2.next = 9;
                break;

              case 6:
                _context2.prev = 6;
                _context2.t0 = _context2['catch'](0);

                console.error('Unable to save configuration: ' + _context2.t0);

              case 9:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[0, 6]]);
      }));

      function saveEnvfile(_x2, _x3) {
        return _ref2.apply(this, arguments);
      }

      return saveEnvfile;
    }()
  }, {
    key: 'prettyPrintAddresses',
    value: function prettyPrintAddresses(contracts) {
      return Object.entries(contracts).forEach(function (_ref3) {
        var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
            key = _ref4[0],
            contract = _ref4[1];

        return console.log('\t' + key + ' -> ' + contract.options.address);
      });
    }
  }, {
    key: 'printSummary',
    value: function printSummary(envFile) {
      console.log('Contracts deployed, save following environment configuration directives to start using them:');
      console.log(envFile);
    }
  }, {
    key: 'contractsToEnvFile',
    value: function contractsToEnvFile(addresses) {
      return 'HEAD_CONTRACT_ADDRESS="' + addresses.head.options.address + '"\nVALIDATOR_SET_CONTRACT_ADDRESS="' + addresses.validatorSet.options.address + '"\nBLOCK_REWARDS_CONTRACT_ADDRESS="' + addresses.blockRewards.options.address + '"';
    }
  }, {
    key: 'help',
    value: function help() {
      return {
        description: 'deploys all contracts'
      };
    }
  }]);
  return DeployTask;
}(_task_base2.default); /*
                        Copyright: Ambrosus Technologies GmbH
                        Email: tech@ambrosus.com
                        
                        This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
                        
                        This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
                        */

exports.default = DeployTask;